;****************************************************************
;
;	Termometro que envia dados de duas temperaturas por porta serie.
;	Usa dois DS1820 e PIC16F628A
;
;		Antonio Sergio Sena		copyright 2005
;		CT2GPW
;		asena@senaeng.com
;
;	v1.0	-	09.10.2005	-	Versao inicial.
;
;****************************************************************
 
	list    p=16f628a, n=500, r=hex, w=-302
	
	include	"p16f628a.inc"


;	Configura os fusiveis do PIC
	__CONFIG	_BOREN_ON & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF & _INTOSC_OSC_NOCLKOUT
	
;****************************************************************

	CBLOCK	0x20		;atribui posicoes na RAM
	count, count1, count2, count3, temp
	W_temp, status_temp
	dsmag,dssign,dezenas,unidades
	tmp0,tmp1,tmp2
	ENDC
	
;****************************************************************



#define		RX_USART	PORTB,1			;recepcao da USART
#define		TX_USART	PORTB,2			;emissao da USART

#define		dsbit		3
#define		dsbit_2		5

#define		BCD		temp













;****************************************************************
;	Vectores de RESET e INTERRUPCAO
;****************************************************************

	org     0x000
	goto	inicio

	org		0x004
	goto	interrupcao

;****************************************************************
;	Rotina de interrupcao:
;
;		- Desliga as interrupcoes TMR0, e recarrega o numero de tons a ler
;
;****************************************************************
interrupcao

	movwf   W_temp          	; save off current W register contents
	swapf	STATUS,W			; move STATUS register into W register
	movwf	status_temp			; save off contents of STATUS register


	swapf	status_temp,W		; retrieve copy of STATUS register
	movwf	STATUS				; restore pre-isr STATUS register 
	swapf   W_temp,F
	swapf   W_temp,W			; restore pre-isr W register contents

	retfie						;sai da interrupcao











;****************************************************************
;****************************************************************
;****************************************************************
;****************************************************************
;	Programa comeca aqui, com as configuracoes do sistema
;****************************************************************
;****************************************************************

inicio
	clrwdt					;limpa watchdog
	clrf	PORTA			;desliga buffers da PORTA A
	clrf	PORTB			;desliga buffers da PORTA B
	movlw	0x07			;desliga comparadores, todas as portas como I/O digitais
	movwf	CMCON

	bsf     STATUS, RP0		;banco 1

	movlw   b'00000000'		;configura a direccao das portas A e B
	movwf   TRISA
	movlw   b'00101100'
	movwf   TRISB

	;Configuracao da USART
	;9k6bps - 25 ; 19k2bps - 12	; 31.25kbps - 7		@4MHz
	clrf	TXSTA
	bsf		TXSTA, BRGH		;liga gerador de timing da USART para alta velocidade
	bsf		TXSTA, TXEN		;liga permissao para transmissao de dados
	movlw	.25				;
	movwf	SPBRG			;carrega valor do temporizador de "baudrate"


	bcf     STATUS, RP0		;banco 0

	clrf	PORTA			;desliga PORTA A por completo
	clrf	PORTB			;desliga PORTA B por completo

	clrf	INTCON			;limpa registo de interrupcoes

	;Configuracao da USART, continuacao
	clrf	RCSTA
	bsf		RCSTA, SPEN		;liga a porta serie



;****************************************************************
;****************************************************************
;	Programa principal
;****************************************************************
;****************************************************************

principal

	call	newtemp		;leitura de nova temperatura

manda_sinais_de_magnitude
negativo	
	movf	dssign, W
	sublw	0xFF
	btfss	STATUS, Z
	 goto	positivo
	movlw	'-'
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	movlw	0xFF			;faz complemento a 2, para converter o resultado negativo para positivo
	xorwf	dsmag, f		; idem
	incf	dsmag, f		; idem
	bcf		STATUS,C
	rrf		dsmag, f		;elimina os 0.5 graus
	goto	manda_temperatura	;manda temperatura, depois de converter de BIN para ASCII

positivo
	movf	dssign, W
	btfss	STATUS, Z
	 goto	manda_temperatura
	movlw	'+'
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	bcf		STATUS,C
	rrf		dsmag, f		;elimina os 0.5 graus


manda_temperatura			;manda temperatura, depois de converter de BIN para ASCII
	call	Bin2ASCII

	movf	dezenas,w
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	movf	unidades,w
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1

manda_espaco
	movlw	' '
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1









	call	newtemp_2		;leitura de nova temperatura

negativo_2
	movf	dssign, W
	sublw	0xFF
	btfss	STATUS, Z
	 goto	positivo_2
	movlw	'-'
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	movlw	0xFF			;faz complemento a 2, para converter o resultado negativo para positivo
	xorwf	dsmag, f		; idem
	incf	dsmag, f		; idem
	bcf		STATUS,C
	rrf		dsmag, f		;elimina os 0.5 graus
	goto	manda_temperatura_2	;manda temperatura, depois de converter de BIN para ASCII

positivo_2
	movf	dssign, W
	btfss	STATUS, Z
	 goto	manda_temperatura_2
	movlw	'+'
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	bcf		STATUS,C
	rrf		dsmag, f		;elimina os 0.5 graus


manda_temperatura_2			;manda temperatura, depois de converter de BIN para ASCII
	call	Bin2ASCII

	movf	dezenas,w
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	movf	unidades,w
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1

manda_mudanca_de_linha_2
	movlw	0x0a
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1
	movlw	0x0d
	movwf	TXREG			;manda para porta serie
	nop						;1 ciclo de espera
	btfss	PIR1, 4			;testa se acabou de mandar byte
	 goto	$-1









	goto	principal

;****************************************************************
;****************************************************************
;	Programa termina aqui
;****************************************************************
;****************************************************************
;****************************************************************
;****************************************************************



















;****************************************************************
;	Inclusao de ficheiros com as rotinas necessarias ao programa
;

	include	"ds1820.asm"		;inclui funcoes do sensor de temperatura :: DS1820 da Dallas






	
	

;*******************************************************************;
;      Converte de Binario para BCD, e de BCD para ASCII
;
;		Output in Dezenas e Unidades
;*******************************************************************

Bin2ASCII
	clrf	unidades
	clrf	dezenas

sub10
	movlw	.10
	subwf	dsmag, f		;sutract 10 minutes to flag_minutos
	btfss	STATUS,C		; <0 ??
	 goto	outsub10
	incf	dezenas, f		;increment in dez_minutos
	goto	sub10			;loop
outsub10
	movlw	.10
	addwf	dsmag, f		;sum 10 to flag_minutos
	bcf		STATUS,C

sub1
	movlw	.1
	subwf	dsmag, f		;subtract 1 flag_minutos
	btfss	STATUS,C		; <0 ??
	 goto	outsub1
	incf	unidades, f		;increment in minutos
	goto	sub1			;loop
outsub1

	movlw	0x30			;adiciona 0x30, para converter para ASCII
	addwf	dezenas, F
	addwf	unidades, F

	return





;****************************************************************
;	Rotina de atraso
;****************************************************************

del_10u
	movlw   .1		;delay 10 usegs
	movwf   count1
	nop
	nop
repeat3
	decfsz  count1,f
	goto    repeat3
	return

del_100u
	movlw   .32		;delay 100 usegs
	movwf   count1
repeat2
	decfsz  count1,f
	goto    repeat2
	return

del_5
	movlw   .48		;delay 5ms
	movwf   count2
delay4
	call    del_100u
	decfsz  count2,f
	goto    delay4
	return

del_500
	movlw	.100		;delay 500ms
	movwf	count3
delay5
	call	del_5
	decfsz	count3, f
	goto	delay5
	return







;****************************************************************
;	End of program
;****************************************************************

	end



















